module jp.ku6.bbs.action.ResponseWriteAction;

import jp.ku6.raptan.core;
import jp.ku6.bbs.constants;
import jp.ku6.bbs.logic.ThreadLogic;
import jp.ku6.bbs.form.ResponseWriteForm;
import jp.ku6.bbs.view.ResponseWriteView;

/++
 + レスポンス書き込み画面
 +/
class ResponseWriteAction
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  logic = スレッドのロジックオブジェクト
	 +/
	this(ThreadLogic logic)
	{
		logic_ = logic;
		form_ = new ResponseWriteForm;
		view_ = new ResponseWriteView(form_);
	}

	/++
	 + メイン処理
	 +
	 + Params:
	 +  request = リクエスト
	 +  response = レスポンス
	 +
	 + Returns:
	 +  内部リダイレクト先のActionのパス
	 +/
	string execute(Request request, Response response)
	{
		// リクエストオブジェクトからフォームのデータを読み出し
		form_.loadFromRequest(request);

		// スレッドIDを取得
		uint threadId = std.conv.to!uint(form_.threadId.value);

		// スレッドIDの指定が無ければトップページに戻す
		if (!logic_.threadExists(threadId))
		{
			response.redirect = APPLICATION_ROOT_URI ~ "/ThreadList.html";
			return null;
		}

		// 初回表示を判定
		if (!form_.submit.isSubmitted)
			form_.reset(); // フォームを初期化
		else
		{
			// 入力エラーを判定
			if (!form_.hasError())
			{
				// 送信されたデータをデータベースに登録
				logic_.writeResponse(threadId, form_.name.value, form_.content.value);

				// スレッド内容ページにリダイレクト
				response.redirect = APPLICATION_ROOT_URI ~ "/ThreadDetail.html?id=" ~ form_.threadId.value;

				return null;
			}
		}

		view_.print(&response.append);

		return null;
	}

private:

	// ロジック
	ThreadLogic logic_;

	// フォーム
	ResponseWriteForm form_;

	// 表示処理
	ResponseWriteView view_;
}

