module jp.ku6.bbs.dao.ResponseDao;

import std.datetime;
import jp.ku6.firebird.core;
import jp.ku6.bbs.entity.ResponseEntity;

/++
 + レスポンスDAO
 +/
class ResponseDao
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  transaction = SQL実行に使用するトランザクションオブジェクト
	 +/
	this(Transaction transaction)
	{
		selectNextSequence_ = new Statement("SELECT GEN_ID(response_sequence, 1) FROM RDB$DATABASE", transaction);
		selectByThreadIdOrderByRegistered_ = new Statement("SELECT id, thread_id, name, content, registered FROM responses WHERE thread_id = ? ORDER BY registered", transaction);
		insert_ = new Statement("INSERT INTO responses(id, thread_id, name, content, registered) VALUES(?, ?, ?, ?, ?)", transaction);
	}

	/++
	 + 次のーケンス番号を取得
	 +
	 + Returns:
	 +  次のシーケンス番号
	 +/
	uint selectNextSequence()
	{
		Statement statement = selectNextSequence_;

		statement.prepare();
		scope(exit) statement.finish();

		auto result = statement.execute();
		return result.fetch() ? result.get!(uint)(0) : 0;
	}

	/++
	 + スレッドIDを指定してレスを全件取得
	 +
	 + Params:
	 +  id = スレッドID
	 +
	 + Returns:
	 +  該当のレス
	 +/
	ResponseEntity[] selectByThreadIdOrderByRegistered(uint id)
	{
		if (id == 0)
			return [];

		Statement statement = selectByThreadIdOrderByRegistered_;

		statement.prepare();
		scope(exit) statement.finish();

		statement.set(0u, id);

		auto result = statement.execute();

		ResponseEntity[] responses = [];
		while (result.fetch())
		{
			auto response = new ResponseEntity;

			response.id = result.get!(uint)(0);
			response.threadId = result.get!(uint)(1);
			response.name = result.get!(string)(2);
			response.content = result.get!(string)(3);
			response.registered = result.get!(SysTime)(4);

			responses ~= response;
		}

		return responses;
	}

	/++
	 + 登録
	 +
	 + Params:
	 +  report = 登録データ
	 +/
	void insert(ResponseEntity entity)
	{
		Statement statement = insert_;

		statement.prepare();
		scope(exit) statement.finish();

		statement.set(0u, entity.id);
		statement.set(1u, entity.threadId);
		statement.set(2u, entity.name);
		statement.set(3u, entity.content);
		statement.set(4u, entity.registered);
		statement.execute();
	}

private:

	// 各SQLオブジェクト
	Statement selectNextSequence_;
	Statement selectByThreadIdOrderByRegistered_;
	Statement insert_;
}
